/* config.h. Handcrafted for Borland C++Builder 6 / Windows 2000 .  */

/* Define to 1 if translation of program messages to the user's native
   language is requested. */
/*#undef ENABLE_NLS*/

/* Define if the GNU dcgettext() function is already present or preinstalled.
   */
/*#undef HAVE_DCGETTEXT*/

/* Define to 1 if you have the <dlfcn.h> header file. */
/*#undef HAVE_DLFCN_H*/

/* Define if the GNU gettext() function is already present or preinstalled. */
/*#undef HAVE_GETTEXT*/

/* Define if you have the iconv() function. */
/*#undef HAVE_ICONV*/

/* Define to 1 if you have the <inttypes.h> header file. */
/*#undef HAVE_INTTYPES_H*/

/* Define to 1 if you have the <memory.h> header file. */
#define HAVE_MEMORY_H 1

/* Define to 1 if you have the <stdint.h> header file. */
#define HAVE_STDINT_H 1

/* Define to 1 if you have the <stdlib.h> header file. */
#define HAVE_STDLIB_H 1

/* Define to 1 if you have the `strcasecmp' function. */
/*#define HAVE_STRCASECMP 1*/

/* Define to 1 if you have the `strdup' function. */
#define HAVE_STRDUP 1

/* Define to 1 if you have the <strings.h> header file. */
/*#undef HAVE_STRINGS_H*/

/* Define to 1 if you have the <string.h> header file. */
#define HAVE_STRING_H 1

/* Define to 1 if you have the <sys/stat.h> header file. */
#undef HAVE_SYS_STAT_H

/* Define to 1 if you have the <sys/types.h> header file. */
#define HAVE_SYS_TYPES_H 1

/* Define to 1 if you have the <unistd.h> header file. */
/*#undef HAVE_UNISTD_H*/

/* Name of package */
#define PACKAGE "confuse"

/* Define to the address where bug reports for this package should be sent. */
#define PACKAGE_BUGREPORT "https://github.com/martinh/libconfuse/issues"

/* Define to the full name of this package. */
#define PACKAGE_NAME "libConfuse"

/* Define to the full name and version of this package. */
#define PACKAGE_STRING "libConfuse 2.5"

/* Define to the one symbol short name of this package. */
#define PACKAGE_TARNAME "confuse"

/* Define to the version of this package. */
#define PACKAGE_VERSION "2.5"

/* Define to 1 if you have the ANSI C header files. */
#define STDC_HEADERS 1

/* Version number of package */
#define VERSION "2.5"

/* Define to 1 if `lex' declares `yytext' as a `char *' by default, not a
   `char[]'. */
/*#undef YYTEXT_POINTER*/

/* Define to empty if `const' does not conform to ANSI C. */
/*#undef const*/

/* Define to 1 if you have the strndup function */
/*#undef HAVE_STRNDUP */

/* Define to 1 if you have the `setenv' function. */
/* #undef HAVE_SETENV */

/* Define to 1 if you have the `unsetenv' function. */
/* #undef HAVE_UNSETENV */

/* Define to 1 if you have the `_putenv' function. */
#define HAVE__PUTENV 1
